"""Generated message classes for translate version v3beta1.

Integrates text translation into your website or application.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'translate'


class BatchDocumentInputConfig(_messages.Message):
  r"""Input configuration for BatchTranslateDocument request.

  Fields:
    gcsSource: Google Cloud Storage location for the source input. This can be
      a single file (for example, `gs://translation-test/input.docx`) or a
      wildcard (for example, `gs://translation-test/*`). File mime type is
      determined based on extension. Supported mime type includes: - `pdf`,
      application/pdf - `docx`, application/vnd.openxmlformats-
      officedocument.wordprocessingml.document - `pptx`,
      application/vnd.openxmlformats-
      officedocument.presentationml.presentation - `xlsx`,
      application/vnd.openxmlformats-officedocument.spreadsheetml.sheet The
      max file size to support for `.docx`, `.pptx` and `.xlsx` is 100MB. The
      max file size to support for `.pdf` is 1GB and the max page limit is
      1000 pages. The max file size to support for all input documents is 1GB.
  """

  gcsSource = _messages.MessageField('GcsSource', 1)


class BatchDocumentOutputConfig(_messages.Message):
  r"""Output configuration for BatchTranslateDocument request.

  Fields:
    gcsDestination: Google Cloud Storage destination for output content. For
      every single input document (for example, gs://a/b/c.[extension]), we
      generate at most 2 * n output files. (n is the # of
      target_language_codes in the BatchTranslateDocumentRequest). While the
      input documents are being processed, we write/update an index file
      `index.csv` under `gcs_destination.output_uri_prefix` (for example,
      gs://translation_output/index.csv) The index file is generated/updated
      as new files are being translated. The format is:
      input_document,target_language_code,translation_output,error_output,
      glossary_translation_output,glossary_error_output `input_document` is
      one file we matched using gcs_source.input_uri. `target_language_code`
      is provided in the request. `translation_output` contains the
      translations. (details provided below) `error_output` contains the error
      message during processing of the file. Both translations_file and
      errors_file could be empty strings if we have no content to output.
      `glossary_translation_output` and `glossary_error_output` are the
      translated output/error when we apply glossaries. They could also be
      empty if we have no content to output. Once a row is present in
      index.csv, the input/output matching never changes. Callers should also
      expect all the content in input_file are processed and ready to be
      consumed (that is, no partial output file is written). Since index.csv
      will be keeping updated during the process, please make sure there is no
      custom retention policy applied on the output bucket that may avoid file
      updating. (https://cloud.google.com/storage/docs/bucket-lock#retention-
      policy) The naming format of translation output files follows (for
      target language code [trg]): `translation_output`:
      `gs://translation_output/a_b_c_[trg]_translation.[extension]`
      `glossary_translation_output`:
      `gs://translation_test/a_b_c_[trg]_glossary_translation.[extension]`.
      The output document will maintain the same file format as the input
      document. The naming format of error output files follows (for target
      language code [trg]): `error_output`:
      `gs://translation_test/a_b_c_[trg]_errors.txt` `glossary_error_output`:
      `gs://translation_test/a_b_c_[trg]_glossary_translation.txt` The error
      output is a txt file containing error details.
  """

  gcsDestination = _messages.MessageField('GcsDestination', 1)


class BatchTranslateDocumentRequest(_messages.Message):
  r"""The BatchTranslateDocument request.

  Messages:
    FormatConversionsValue: Optional. File format conversion map to be applied
      to all input files. Map's key is the original mime_type. Map's value is
      the target mime_type of translated documents. Supported file format
      conversion includes: - `application/pdf` to
      `application/vnd.openxmlformats-
      officedocument.wordprocessingml.document` If nothing specified, output
      files will be in the same format as the original file.
    GlossariesValue: Optional. Glossaries to be applied. It's keyed by target
      language code.
    ModelsValue: Optional. The models to use for translation. Map's key is
      target language code. Map's value is the model name. Value can be a
      built-in general model, or an AutoML Translation model. The value format
      depends on model type: - AutoML Translation models: `projects/{project-
      number-or-id}/locations/{location-id}/models/{model-id}` - General
      (built-in) models: `projects/{project-number-or-id}/locations/{location-
      id}/models/general/nmt`, If the map is empty or a specific model is not
      requested for a language pair, then default google model (nmt) is used.

  Fields:
    customizedAttribution: Optional. This flag is to support user customized
      attribution. If not provided, the default is `Machine Translated by
      Google`. Customized attribution should follow rules in
      https://cloud.google.com/translate/attribution#attribution_and_logos
    enableRotationCorrection: Optional. If true, enable auto rotation
      correction in DVS.
    enableShadowRemovalNativePdf: Optional. If true, use the text removal
      server to remove the shadow text on background image for native pdf
      translation. Shadow removal feature can only be enabled when
      is_translate_native_pdf_only: false && pdf_native_only: false
    formatConversions: Optional. File format conversion map to be applied to
      all input files. Map's key is the original mime_type. Map's value is the
      target mime_type of translated documents. Supported file format
      conversion includes: - `application/pdf` to
      `application/vnd.openxmlformats-
      officedocument.wordprocessingml.document` If nothing specified, output
      files will be in the same format as the original file.
    glossaries: Optional. Glossaries to be applied. It's keyed by target
      language code.
    inputConfigs: Required. Input configurations. The total number of files
      matched should be <= 100. The total content size to translate should be
      <= 100M Unicode codepoints. The files must use UTF-8 encoding.
    models: Optional. The models to use for translation. Map's key is target
      language code. Map's value is the model name. Value can be a built-in
      general model, or an AutoML Translation model. The value format depends
      on model type: - AutoML Translation models: `projects/{project-number-
      or-id}/locations/{location-id}/models/{model-id}` - General (built-in)
      models: `projects/{project-number-or-id}/locations/{location-
      id}/models/general/nmt`, If the map is empty or a specific model is not
      requested for a language pair, then default google model (nmt) is used.
    outputConfig: Required. Output configuration. If 2 input configs match to
      the same file (that is, same input path), we don't generate output for
      duplicate inputs.
    pdfNativeOnly: Optional. If true, only native pdf pages will be
      translated.
    sourceLanguageCode: Required. The BCP-47 language code of the input
      document if known, for example, "en-US" or "sr-Latn". Supported language
      codes are listed in [Language
      Support](https://cloud.google.com/translate/docs/languages).
    targetLanguageCodes: Required. The BCP-47 language code to use for
      translation of the input document. Specify up to 10 language codes here.
      Supported language codes are listed in [Language
      Support](https://cloud.google.com/translate/docs/languages).
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class FormatConversionsValue(_messages.Message):
    r"""Optional. File format conversion map to be applied to all input files.
    Map's key is the original mime_type. Map's value is the target mime_type
    of translated documents. Supported file format conversion includes: -
    `application/pdf` to `application/vnd.openxmlformats-
    officedocument.wordprocessingml.document` If nothing specified, output
    files will be in the same format as the original file.

    Messages:
      AdditionalProperty: An additional property for a FormatConversionsValue
        object.

    Fields:
      additionalProperties: Additional properties of type
        FormatConversionsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a FormatConversionsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class GlossariesValue(_messages.Message):
    r"""Optional. Glossaries to be applied. It's keyed by target language
    code.

    Messages:
      AdditionalProperty: An additional property for a GlossariesValue object.

    Fields:
      additionalProperties: Additional properties of type GlossariesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a GlossariesValue object.

      Fields:
        key: Name of the additional property.
        value: A TranslateTextGlossaryConfig attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('TranslateTextGlossaryConfig', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ModelsValue(_messages.Message):
    r"""Optional. The models to use for translation. Map's key is target
    language code. Map's value is the model name. Value can be a built-in
    general model, or an AutoML Translation model. The value format depends on
    model type: - AutoML Translation models: `projects/{project-number-or-
    id}/locations/{location-id}/models/{model-id}` - General (built-in)
    models: `projects/{project-number-or-id}/locations/{location-
    id}/models/general/nmt`, If the map is empty or a specific model is not
    requested for a language pair, then default google model (nmt) is used.

    Messages:
      AdditionalProperty: An additional property for a ModelsValue object.

    Fields:
      additionalProperties: Additional properties of type ModelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ModelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  customizedAttribution = _messages.StringField(1)
  enableRotationCorrection = _messages.BooleanField(2)
  enableShadowRemovalNativePdf = _messages.BooleanField(3)
  formatConversions = _messages.MessageField('FormatConversionsValue', 4)
  glossaries = _messages.MessageField('GlossariesValue', 5)
  inputConfigs = _messages.MessageField('BatchDocumentInputConfig', 6, repeated=True)
  models = _messages.MessageField('ModelsValue', 7)
  outputConfig = _messages.MessageField('BatchDocumentOutputConfig', 8)
  pdfNativeOnly = _messages.BooleanField(9)
  sourceLanguageCode = _messages.StringField(10)
  targetLanguageCodes = _messages.StringField(11, repeated=True)


class BatchTranslateTextRequest(_messages.Message):
  r"""The batch translation request.

  Messages:
    GlossariesValue: Optional. Glossaries to be applied for translation. It's
      keyed by target language code.
    LabelsValue: Optional. The labels with user-defined metadata for the
      request. Label keys and values can be no longer than 63 characters
      (Unicode codepoints), can only contain lowercase letters, numeric
      characters, underscores and dashes. International characters are
      allowed. Label values are optional. Label keys must start with a letter.
      See https://cloud.google.com/translate/docs/labels for more information.
    ModelsValue: Optional. The models to use for translation. Map's key is
      target language code. Map's value is model name. Value can be a built-in
      general model, or an AutoML Translation model. The value format depends
      on model type: - AutoML Translation models: `projects/{project-number-
      or-id}/locations/{location-id}/models/{model-id}` - General (built-in)
      models: `projects/{project-number-or-id}/locations/{location-
      id}/models/general/nmt`, If the map is empty or a specific model is not
      requested for a language pair, then default google model (nmt) is used.

  Fields:
    glossaries: Optional. Glossaries to be applied for translation. It's keyed
      by target language code.
    inputConfigs: Required. Input configurations. The total number of files
      matched should be <= 100. The total content size should be <= 100M
      Unicode codepoints. The files must use UTF-8 encoding.
    labels: Optional. The labels with user-defined metadata for the request.
      Label keys and values can be no longer than 63 characters (Unicode
      codepoints), can only contain lowercase letters, numeric characters,
      underscores and dashes. International characters are allowed. Label
      values are optional. Label keys must start with a letter. See
      https://cloud.google.com/translate/docs/labels for more information.
    models: Optional. The models to use for translation. Map's key is target
      language code. Map's value is model name. Value can be a built-in
      general model, or an AutoML Translation model. The value format depends
      on model type: - AutoML Translation models: `projects/{project-number-
      or-id}/locations/{location-id}/models/{model-id}` - General (built-in)
      models: `projects/{project-number-or-id}/locations/{location-
      id}/models/general/nmt`, If the map is empty or a specific model is not
      requested for a language pair, then default google model (nmt) is used.
    outputConfig: Required. Output configuration. If 2 input configs match to
      the same file (that is, same input path), we don't generate output for
      duplicate inputs.
    sourceLanguageCode: Required. Source language code. Supported language
      codes are listed in [Language
      Support](https://cloud.google.com/translate/docs/languages).
    targetLanguageCodes: Required. Specify up to 10 language codes here.
      Supported language codes are listed in [Language
      Support](https://cloud.google.com/translate/docs/languages).
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class GlossariesValue(_messages.Message):
    r"""Optional. Glossaries to be applied for translation. It's keyed by
    target language code.

    Messages:
      AdditionalProperty: An additional property for a GlossariesValue object.

    Fields:
      additionalProperties: Additional properties of type GlossariesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a GlossariesValue object.

      Fields:
        key: Name of the additional property.
        value: A TranslateTextGlossaryConfig attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('TranslateTextGlossaryConfig', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. The labels with user-defined metadata for the request. Label
    keys and values can be no longer than 63 characters (Unicode codepoints),
    can only contain lowercase letters, numeric characters, underscores and
    dashes. International characters are allowed. Label values are optional.
    Label keys must start with a letter. See
    https://cloud.google.com/translate/docs/labels for more information.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ModelsValue(_messages.Message):
    r"""Optional. The models to use for translation. Map's key is target
    language code. Map's value is model name. Value can be a built-in general
    model, or an AutoML Translation model. The value format depends on model
    type: - AutoML Translation models: `projects/{project-number-or-
    id}/locations/{location-id}/models/{model-id}` - General (built-in)
    models: `projects/{project-number-or-id}/locations/{location-
    id}/models/general/nmt`, If the map is empty or a specific model is not
    requested for a language pair, then default google model (nmt) is used.

    Messages:
      AdditionalProperty: An additional property for a ModelsValue object.

    Fields:
      additionalProperties: Additional properties of type ModelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ModelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  glossaries = _messages.MessageField('GlossariesValue', 1)
  inputConfigs = _messages.MessageField('InputConfig', 2, repeated=True)
  labels = _messages.MessageField('LabelsValue', 3)
  models = _messages.MessageField('ModelsValue', 4)
  outputConfig = _messages.MessageField('OutputConfig', 5)
  sourceLanguageCode = _messages.StringField(6)
  targetLanguageCodes = _messages.StringField(7, repeated=True)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class DetectLanguageRequest(_messages.Message):
  r"""The request message for language detection.

  Messages:
    LabelsValue: Optional. The labels with user-defined metadata for the
      request. Label keys and values can be no longer than 63 characters
      (Unicode codepoints), can only contain lowercase letters, numeric
      characters, underscores and dashes. International characters are
      allowed. Label values are optional. Label keys must start with a letter.
      See https://cloud.google.com/translate/docs/labels for more information.

  Fields:
    content: The content of the input stored as a string.
    labels: Optional. The labels with user-defined metadata for the request.
      Label keys and values can be no longer than 63 characters (Unicode
      codepoints), can only contain lowercase letters, numeric characters,
      underscores and dashes. International characters are allowed. Label
      values are optional. Label keys must start with a letter. See
      https://cloud.google.com/translate/docs/labels for more information.
    mimeType: Optional. The format of the source text, for example,
      "text/html", "text/plain". If left blank, the MIME type defaults to
      "text/html".
    model: Optional. The language detection model to be used. Format:
      `projects/{project-number-or-id}/locations/{location-
      id}/models/language-detection/{model-id}` Only one language detection
      model is currently supported: `projects/{project-number-or-
      id}/locations/{location-id}/models/language-detection/default`. If not
      specified, the default model is used.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. The labels with user-defined metadata for the request. Label
    keys and values can be no longer than 63 characters (Unicode codepoints),
    can only contain lowercase letters, numeric characters, underscores and
    dashes. International characters are allowed. Label values are optional.
    Label keys must start with a letter. See
    https://cloud.google.com/translate/docs/labels for more information.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  content = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  mimeType = _messages.StringField(3)
  model = _messages.StringField(4)


class DetectLanguageResponse(_messages.Message):
  r"""The response message for language detection.

  Fields:
    languages: A list of detected languages sorted by detection confidence in
      descending order. The most probable language first.
  """

  languages = _messages.MessageField('DetectedLanguage', 1, repeated=True)


class DetectedLanguage(_messages.Message):
  r"""The response message for language detection.

  Fields:
    confidence: The confidence of the detection result for this language.
    languageCode: The BCP-47 language code of source content in the request,
      detected automatically.
  """

  confidence = _messages.FloatField(1, variant=_messages.Variant.FLOAT)
  languageCode = _messages.StringField(2)


class DocumentInputConfig(_messages.Message):
  r"""A document translation request input config.

  Fields:
    content: Document's content represented as a stream of bytes.
    gcsSource: Google Cloud Storage location. This must be a single file. For
      example: gs://example_bucket/example_file.pdf
    mimeType: Specifies the input document's mime_type. If not specified it
      will be determined using the file extension for gcs_source provided
      files. For a file provided through bytes content the mime_type must be
      provided. Currently supported mime types are: - application/pdf -
      application/vnd.openxmlformats-officedocument.wordprocessingml.document
      - application/vnd.openxmlformats-
      officedocument.presentationml.presentation -
      application/vnd.openxmlformats-officedocument.spreadsheetml.sheet
  """

  content = _messages.BytesField(1)
  gcsSource = _messages.MessageField('GcsSource', 2)
  mimeType = _messages.StringField(3)


class DocumentOutputConfig(_messages.Message):
  r"""A document translation request output config.

  Fields:
    gcsDestination: Optional. Google Cloud Storage destination for the
      translation output, e.g., `gs://my_bucket/my_directory/`. The
      destination directory provided does not have to be empty, but the bucket
      must exist. If a file with the same name as the output file already
      exists in the destination an error will be returned. For a
      DocumentInputConfig.contents provided document, the output file will
      have the name "output_[trg]_translations.[ext]", where - [trg]
      corresponds to the translated file's language code, - [ext] corresponds
      to the translated file's extension according to its mime type. For a
      DocumentInputConfig.gcs_uri provided document, the output file will have
      a name according to its URI. For example: an input file with URI:
      `gs://a/b/c.[extension]` stored in a gcs_destination bucket with name
      "my_bucket" will have an output URI:
      `gs://my_bucket/a_b_c_[trg]_translations.[ext]`, where - [trg]
      corresponds to the translated file's language code, - [ext] corresponds
      to the translated file's extension according to its mime type. If the
      document was directly provided through the request, then the output
      document will have the format:
      `gs://my_bucket/translated_document_[trg]_translations.[ext]`, where -
      [trg] corresponds to the translated file's language code, - [ext]
      corresponds to the translated file's extension according to its mime
      type. If a glossary was provided, then the output URI for the glossary
      translation will be equal to the default output URI but have
      `glossary_translations` instead of `translations`. For the previous
      example, its glossary URI would be:
      `gs://my_bucket/a_b_c_[trg]_glossary_translations.[ext]`. Thus the max
      number of output files will be 2 (Translated document, Glossary
      translated document). Callers should expect no partial outputs. If there
      is any error during document translation, no output will be stored in
      the Cloud Storage bucket.
    mimeType: Optional. Specifies the translated document's mime_type. If not
      specified, the translated file's mime type will be the same as the input
      file's mime type. Currently only support the output mime type to be the
      same as input mime type. - application/pdf -
      application/vnd.openxmlformats-officedocument.wordprocessingml.document
      - application/vnd.openxmlformats-
      officedocument.presentationml.presentation -
      application/vnd.openxmlformats-officedocument.spreadsheetml.sheet
  """

  gcsDestination = _messages.MessageField('GcsDestination', 1)
  mimeType = _messages.StringField(2)


class DocumentTranslation(_messages.Message):
  r"""A translated document message.

  Fields:
    byteStreamOutputs: The array of translated documents. It is expected to be
      size 1 for now. We may produce multiple translated documents in the
      future for other type of file formats.
    detectedLanguageCode: The detected language for the input document. If the
      user did not provide the source language for the input document, this
      field will have the language code automatically detected. If the source
      language was passed, auto-detection of the language does not occur and
      this field is empty.
    mimeType: The translated document's mime type.
  """

  byteStreamOutputs = _messages.BytesField(1, repeated=True)
  detectedLanguageCode = _messages.StringField(2)
  mimeType = _messages.StringField(3)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class GcsDestination(_messages.Message):
  r"""The Google Cloud Storage location for the output content.

  Fields:
    outputUriPrefix: Required. There must be no files under
      'output_uri_prefix'. 'output_uri_prefix' must end with "/" and start
      with "gs://", otherwise an INVALID_ARGUMENT (400) error is returned.
  """

  outputUriPrefix = _messages.StringField(1)


class GcsSource(_messages.Message):
  r"""The Google Cloud Storage location for the input content.

  Fields:
    inputUri: Required. Source data URI. For example,
      `gs://my_bucket/my_object`.
  """

  inputUri = _messages.StringField(1)


class Glossary(_messages.Message):
  r"""Represents a glossary built from user provided data.

  Fields:
    endTime: Output only. When the glossary creation was finished.
    entryCount: Output only. The number of entries defined in the glossary.
    inputConfig: Required. Provides examples to build the glossary from. Total
      glossary must not exceed 10M Unicode codepoints.
    languageCodesSet: Used with equivalent term set glossaries.
    languagePair: Used with unidirectional glossaries.
    name: Required. The resource name of the glossary. Glossary names have the
      form `projects/{project-number-or-id}/locations/{location-
      id}/glossaries/{glossary-id}`.
    submitTime: Output only. When CreateGlossary was called.
  """

  endTime = _messages.StringField(1)
  entryCount = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  inputConfig = _messages.MessageField('GlossaryInputConfig', 3)
  languageCodesSet = _messages.MessageField('LanguageCodesSet', 4)
  languagePair = _messages.MessageField('LanguageCodePair', 5)
  name = _messages.StringField(6)
  submitTime = _messages.StringField(7)


class GlossaryInputConfig(_messages.Message):
  r"""Input configuration for glossaries.

  Fields:
    gcsSource: Required. Google Cloud Storage location of glossary data. File
      format is determined based on the filename extension. API returns
      [google.rpc.Code.INVALID_ARGUMENT] for unsupported URI-s and file
      formats. Wildcards are not allowed. This must be a single file in one of
      the following formats: For unidirectional glossaries: - TSV/CSV
      (`.tsv`/`.csv`): 2 column file, tab- or comma-separated. The first
      column is source text. The second column is target text. The file must
      not contain headers. That is, the first row is data, not column names. -
      TMX (`.tmx`): TMX file with parallel data defining source/target term
      pairs. For equivalent term sets glossaries: - CSV (`.csv`): Multi-column
      CSV file defining equivalent glossary terms in multiple languages. See
      documentation for more information -
      [glossaries](https://cloud.google.com/translate/docs/advanced/glossary).
  """

  gcsSource = _messages.MessageField('GcsSource', 1)


class InputConfig(_messages.Message):
  r"""Input configuration for BatchTranslateText request.

  Fields:
    gcsSource: Required. Google Cloud Storage location for the source input.
      This can be a single file (for example, `gs://translation-
      test/input.tsv`) or a wildcard (for example, `gs://translation-test/*`).
      If a file extension is `.tsv`, it can contain either one or two columns.
      The first column (optional) is the id of the text request. If the first
      column is missing, we use the row number (0-based) from the input file
      as the ID in the output file. The second column is the actual text to be
      translated. We recommend each row be <= 10K Unicode codepoints,
      otherwise an error might be returned. Note that the input tsv must be
      RFC 4180 compliant. You could use https://github.com/Clever/csvlint to
      check potential formatting errors in your tsv file. csvlint
      --delimiter='\t' your_input_file.tsv The other supported file extensions
      are `.txt` or `.html`, which is treated as a single large chunk of text.
    mimeType: Optional. Can be "text/plain" or "text/html". For `.tsv`,
      "text/html" is used if mime_type is missing. For `.html`, this field
      must be "text/html" or empty. For `.txt`, this field must be
      "text/plain" or empty.
  """

  gcsSource = _messages.MessageField('GcsSource', 1)
  mimeType = _messages.StringField(2)


class LanguageCodePair(_messages.Message):
  r"""Used with unidirectional glossaries.

  Fields:
    sourceLanguageCode: Required. The BCP-47 language code of the input text,
      for example, "en-US". Expected to be an exact match for
      GlossaryTerm.language_code.
    targetLanguageCode: Required. The BCP-47 language code for translation
      output, for example, "zh-CN". Expected to be an exact match for
      GlossaryTerm.language_code.
  """

  sourceLanguageCode = _messages.StringField(1)
  targetLanguageCode = _messages.StringField(2)


class LanguageCodesSet(_messages.Message):
  r"""Used with equivalent term set glossaries.

  Fields:
    languageCodes: The BCP-47 language code(s) for terms defined in the
      glossary. All entries are unique. The list contains at least two
      entries. Expected to be an exact match for GlossaryTerm.language_code.
  """

  languageCodes = _messages.StringField(1, repeated=True)


class ListGlossariesResponse(_messages.Message):
  r"""Response message for ListGlossaries.

  Fields:
    glossaries: The list of glossaries for a project.
    nextPageToken: A token to retrieve a page of results. Pass this value in
      the [ListGlossariesRequest.page_token] field in the subsequent call to
      `ListGlossaries` method to retrieve the next page of results.
  """

  glossaries = _messages.MessageField('Glossary', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OutputConfig(_messages.Message):
  r"""Output configuration for BatchTranslateText request.

  Fields:
    gcsDestination: Google Cloud Storage destination for output content. For
      every single input file (for example, gs://a/b/c.[extension]), we
      generate at most 2 * n output files. (n is the # of
      target_language_codes in the BatchTranslateTextRequest). Output files
      (tsv) generated are compliant with RFC 4180 except that record
      delimiters are '\n' instead of '\r\n'. We don't provide any way to
      change record delimiters. While the input files are being processed, we
      write/update an index file 'index.csv' under 'output_uri_prefix' (for
      example, gs://translation-test/index.csv) The index file is
      generated/updated as new files are being translated. The format is:
      input_file,target_language_code,translations_file,errors_file,
      glossary_translations_file,glossary_errors_file input_file is one file
      we matched using gcs_source.input_uri. target_language_code is provided
      in the request. translations_file contains the translations. (details
      provided below) errors_file contains the errors during processing of the
      file. (details below). Both translations_file and errors_file could be
      empty strings if we have no content to output.
      glossary_translations_file and glossary_errors_file are always empty
      strings if the input_file is tsv. They could also be empty if we have no
      content to output. Once a row is present in index.csv, the input/output
      matching never changes. Callers should also expect all the content in
      input_file are processed and ready to be consumed (that is, no partial
      output file is written). Since index.csv will be keeping updated during
      the process, please make sure there is no custom retention policy
      applied on the output bucket that may avoid file updating.
      (https://cloud.google.com/storage/docs/bucket-lock#retention-policy) The
      format of translations_file (for target language code 'trg') is:
      `gs://translation_test/a_b_c_'trg'_translations.[extension]` If the
      input file extension is tsv, the output has the following columns:
      Column 1: ID of the request provided in the input, if it's not provided
      in the input, then the input row number is used (0-based). Column 2:
      source sentence. Column 3: translation without applying a glossary.
      Empty string if there is an error. Column 4 (only present if a glossary
      is provided in the request): translation after applying the glossary.
      Empty string if there is an error applying the glossary. Could be same
      string as column 3 if there is no glossary applied. If input file
      extension is a txt or html, the translation is directly written to the
      output file. If glossary is requested, a separate
      glossary_translations_file has format of
      `gs://translation_test/a_b_c_'trg'_glossary_translations.[extension]`
      The format of errors file (for target language code 'trg') is:
      `gs://translation_test/a_b_c_'trg'_errors.[extension]` If the input file
      extension is tsv, errors_file contains the following: Column 1: ID of
      the request provided in the input, if it's not provided in the input,
      then the input row number is used (0-based). Column 2: source sentence.
      Column 3: Error detail for the translation. Could be empty. Column 4
      (only present if a glossary is provided in the request): Error when
      applying the glossary. If the input file extension is txt or html,
      glossary_error_file will be generated that contains error details.
      glossary_error_file has format of
      `gs://translation_test/a_b_c_'trg'_glossary_errors.[extension]`
  """

  gcsDestination = _messages.MessageField('GcsDestination', 1)


class RefineTextRequest(_messages.Message):
  r"""Request message for RefineText.

  Fields:
    refinementEntries: Required. The source texts and original translations in
      the source and target languages.
    sourceLanguageCode: Required. The BCP-47 language code of the source text
      in the request, for example, "en-US".
    targetLanguageCode: Required. The BCP-47 language code for translation
      output, for example, "zh-CN".
  """

  refinementEntries = _messages.MessageField('RefinementEntry', 1, repeated=True)
  sourceLanguageCode = _messages.StringField(2)
  targetLanguageCode = _messages.StringField(3)


class RefineTextResponse(_messages.Message):
  r"""Response message for RefineText.

  Fields:
    refinedTranslations: The refined translations obtained from the original
      translations.
  """

  refinedTranslations = _messages.StringField(1, repeated=True)


class RefinementEntry(_messages.Message):
  r"""A single refinement entry for RefineTextRequest.

  Fields:
    originalTranslation: Required. The original translation of the source
      text.
    sourceText: Required. The source text to be refined.
  """

  originalTranslation = _messages.StringField(1)
  sourceText = _messages.StringField(2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class SupportedLanguage(_messages.Message):
  r"""A single supported language response corresponds to information related
  to one supported language.

  Fields:
    displayName: Human readable name of the language localized in the display
      language specified in the request.
    languageCode: Supported language code, generally consisting of its ISO
      639-1 identifier, for example, 'en', 'ja'. In certain cases, BCP-47
      codes including language and region identifiers are returned (for
      example, 'zh-TW' and 'zh-CN')
    supportSource: Can be used as source language.
    supportTarget: Can be used as target language.
  """

  displayName = _messages.StringField(1)
  languageCode = _messages.StringField(2)
  supportSource = _messages.BooleanField(3)
  supportTarget = _messages.BooleanField(4)


class SupportedLanguages(_messages.Message):
  r"""The response message for discovering supported languages.

  Fields:
    languages: A list of supported language responses. This list contains an
      entry for each language the Translation API supports.
  """

  languages = _messages.MessageField('SupportedLanguage', 1, repeated=True)


class TranslateDocumentRequest(_messages.Message):
  r"""A document translation request.

  Messages:
    LabelsValue: Optional. The labels with user-defined metadata for the
      request. Label keys and values can be no longer than 63 characters
      (Unicode codepoints), can only contain lowercase letters, numeric
      characters, underscores and dashes. International characters are
      allowed. Label values are optional. Label keys must start with a letter.
      See https://cloud.google.com/translate/docs/advanced/labels for more
      information.

  Fields:
    customizedAttribution: Optional. This flag is to support user customized
      attribution. If not provided, the default is `Machine Translated by
      Google`. Customized attribution should follow rules in
      https://cloud.google.com/translate/attribution#attribution_and_logos
    documentInputConfig: Required. Input configurations.
    documentOutputConfig: Optional. Output configurations. Defines if the
      output file should be stored within Cloud Storage as well as the desired
      output format. If not provided the translated file will only be returned
      through a byte-stream and its output mime type will be the same as the
      input file's mime type.
    enableRotationCorrection: Optional. If true, enable auto rotation
      correction in DVS.
    enableShadowRemovalNativePdf: Optional. If true, use the text removal
      server to remove the shadow text on background image for native pdf
      translation. Shadow removal feature can only be enabled when
      is_translate_native_pdf_only: false && pdf_native_only: false
    glossaryConfig: Optional. Glossary to be applied. The glossary must be
      within the same region (have the same location-id) as the model,
      otherwise an INVALID_ARGUMENT (400) error is returned.
    isTranslateNativePdfOnly: Optional. is_translate_native_pdf_only field for
      external customers. If true, the page limit of online native pdf
      translation is 300 and only native pdf pages will be translated.
    labels: Optional. The labels with user-defined metadata for the request.
      Label keys and values can be no longer than 63 characters (Unicode
      codepoints), can only contain lowercase letters, numeric characters,
      underscores and dashes. International characters are allowed. Label
      values are optional. Label keys must start with a letter. See
      https://cloud.google.com/translate/docs/advanced/labels for more
      information.
    model: Optional. The `model` type requested for this translation. The
      format depends on model type: - AutoML Translation models:
      `projects/{project-number-or-id}/locations/{location-id}/models/{model-
      id}` - General (built-in) models: `projects/{project-number-or-
      id}/locations/{location-id}/models/general/nmt`, If not provided, the
      default Google model (NMT) will be used for translation.
    sourceLanguageCode: Optional. The BCP-47 language code of the input
      document if known, for example, "en-US" or "sr-Latn". Supported language
      codes are listed in [Language
      Support](https://cloud.google.com/translate/docs/languages). If the
      source language isn't specified, the API attempts to identify the source
      language automatically and returns the source language within the
      response. Source language must be specified if the request contains a
      glossary or a custom model.
    targetLanguageCode: Required. The BCP-47 language code to use for
      translation of the input document, set to one of the language codes
      listed in [Language
      Support](https://cloud.google.com/translate/docs/languages).
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. The labels with user-defined metadata for the request. Label
    keys and values can be no longer than 63 characters (Unicode codepoints),
    can only contain lowercase letters, numeric characters, underscores and
    dashes. International characters are allowed. Label values are optional.
    Label keys must start with a letter. See
    https://cloud.google.com/translate/docs/advanced/labels for more
    information.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  customizedAttribution = _messages.StringField(1)
  documentInputConfig = _messages.MessageField('DocumentInputConfig', 2)
  documentOutputConfig = _messages.MessageField('DocumentOutputConfig', 3)
  enableRotationCorrection = _messages.BooleanField(4)
  enableShadowRemovalNativePdf = _messages.BooleanField(5)
  glossaryConfig = _messages.MessageField('TranslateTextGlossaryConfig', 6)
  isTranslateNativePdfOnly = _messages.BooleanField(7)
  labels = _messages.MessageField('LabelsValue', 8)
  model = _messages.StringField(9)
  sourceLanguageCode = _messages.StringField(10)
  targetLanguageCode = _messages.StringField(11)


class TranslateDocumentResponse(_messages.Message):
  r"""A translated document response message.

  Fields:
    documentTranslation: Translated document.
    glossaryConfig: The `glossary_config` used for this translation.
    glossaryDocumentTranslation: The document's translation output if a
      glossary is provided in the request. This can be the same as
      [TranslateDocumentResponse.document_translation] if no glossary terms
      apply.
    model: Only present when 'model' is present in the request. 'model' is
      normalized to have a project number. For example: If the 'model' field
      in TranslateDocumentRequest is: `projects/{project-
      id}/locations/{location-id}/models/general/nmt` then `model` here would
      be normalized to `projects/{project-number}/locations/{location-
      id}/models/general/nmt`.
  """

  documentTranslation = _messages.MessageField('DocumentTranslation', 1)
  glossaryConfig = _messages.MessageField('TranslateTextGlossaryConfig', 2)
  glossaryDocumentTranslation = _messages.MessageField('DocumentTranslation', 3)
  model = _messages.StringField(4)


class TranslateProjectsDetectLanguageRequest(_messages.Message):
  r"""A TranslateProjectsDetectLanguageRequest object.

  Fields:
    detectLanguageRequest: A DetectLanguageRequest resource to be passed as
      the request body.
    parent: Required. Project or location to make a call. Must refer to a
      caller's project. Format: `projects/{project-number-or-
      id}/locations/{location-id}` or `projects/{project-number-or-id}`. For
      global calls, use `projects/{project-number-or-id}/locations/global` or
      `projects/{project-number-or-id}`. Only models within the same region
      (has same location-id) can be used. Otherwise an INVALID_ARGUMENT (400)
      error is returned.
  """

  detectLanguageRequest = _messages.MessageField('DetectLanguageRequest', 1)
  parent = _messages.StringField(2, required=True)


class TranslateProjectsGetSupportedLanguagesRequest(_messages.Message):
  r"""A TranslateProjectsGetSupportedLanguagesRequest object.

  Fields:
    displayLanguageCode: Optional. The language to use to return localized,
      human readable names of supported languages. If missing, then display
      names are not returned in a response.
    model: Optional. Get supported languages of this model. The format depends
      on model type: - AutoML Translation models: `projects/{project-number-
      or-id}/locations/{location-id}/models/{model-id}` - General (built-in)
      models: `projects/{project-number-or-id}/locations/{location-
      id}/models/general/nmt`, Returns languages supported by the specified
      model. If missing, we get supported languages of Google general NMT
      model.
    parent: Required. Project or location to make a call. Must refer to a
      caller's project. Format: `projects/{project-number-or-id}` or
      `projects/{project-number-or-id}/locations/{location-id}`. For global
      calls, use `projects/{project-number-or-id}/locations/global` or
      `projects/{project-number-or-id}`. Non-global location is required for
      AutoML models. Only models within the same region (have same location-
      id) can be used, otherwise an INVALID_ARGUMENT (400) error is returned.
  """

  displayLanguageCode = _messages.StringField(1)
  model = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class TranslateProjectsLocationsBatchTranslateDocumentRequest(_messages.Message):
  r"""A TranslateProjectsLocationsBatchTranslateDocumentRequest object.

  Fields:
    batchTranslateDocumentRequest: A BatchTranslateDocumentRequest resource to
      be passed as the request body.
    parent: Required. Location to make a regional call. Format:
      `projects/{project-number-or-id}/locations/{location-id}`. The `global`
      location is not supported for batch translation. Only AutoML Translation
      models or glossaries within the same region (have the same location-id)
      can be used, otherwise an INVALID_ARGUMENT (400) error is returned.
  """

  batchTranslateDocumentRequest = _messages.MessageField('BatchTranslateDocumentRequest', 1)
  parent = _messages.StringField(2, required=True)


class TranslateProjectsLocationsBatchTranslateTextRequest(_messages.Message):
  r"""A TranslateProjectsLocationsBatchTranslateTextRequest object.

  Fields:
    batchTranslateTextRequest: A BatchTranslateTextRequest resource to be
      passed as the request body.
    parent: Required. Location to make a call. Must refer to a caller's
      project. Format: `projects/{project-number-or-id}/locations/{location-
      id}`. The `global` location is not supported for batch translation. Only
      AutoML Translation models or glossaries within the same region (have the
      same location-id) can be used, otherwise an INVALID_ARGUMENT (400) error
      is returned.
  """

  batchTranslateTextRequest = _messages.MessageField('BatchTranslateTextRequest', 1)
  parent = _messages.StringField(2, required=True)


class TranslateProjectsLocationsDetectLanguageRequest(_messages.Message):
  r"""A TranslateProjectsLocationsDetectLanguageRequest object.

  Fields:
    detectLanguageRequest: A DetectLanguageRequest resource to be passed as
      the request body.
    parent: Required. Project or location to make a call. Must refer to a
      caller's project. Format: `projects/{project-number-or-
      id}/locations/{location-id}` or `projects/{project-number-or-id}`. For
      global calls, use `projects/{project-number-or-id}/locations/global` or
      `projects/{project-number-or-id}`. Only models within the same region
      (has same location-id) can be used. Otherwise an INVALID_ARGUMENT (400)
      error is returned.
  """

  detectLanguageRequest = _messages.MessageField('DetectLanguageRequest', 1)
  parent = _messages.StringField(2, required=True)


class TranslateProjectsLocationsGetRequest(_messages.Message):
  r"""A TranslateProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class TranslateProjectsLocationsGetSupportedLanguagesRequest(_messages.Message):
  r"""A TranslateProjectsLocationsGetSupportedLanguagesRequest object.

  Fields:
    displayLanguageCode: Optional. The language to use to return localized,
      human readable names of supported languages. If missing, then display
      names are not returned in a response.
    model: Optional. Get supported languages of this model. The format depends
      on model type: - AutoML Translation models: `projects/{project-number-
      or-id}/locations/{location-id}/models/{model-id}` - General (built-in)
      models: `projects/{project-number-or-id}/locations/{location-
      id}/models/general/nmt`, Returns languages supported by the specified
      model. If missing, we get supported languages of Google general NMT
      model.
    parent: Required. Project or location to make a call. Must refer to a
      caller's project. Format: `projects/{project-number-or-id}` or
      `projects/{project-number-or-id}/locations/{location-id}`. For global
      calls, use `projects/{project-number-or-id}/locations/global` or
      `projects/{project-number-or-id}`. Non-global location is required for
      AutoML models. Only models within the same region (have same location-
      id) can be used, otherwise an INVALID_ARGUMENT (400) error is returned.
  """

  displayLanguageCode = _messages.StringField(1)
  model = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class TranslateProjectsLocationsGlossariesCreateRequest(_messages.Message):
  r"""A TranslateProjectsLocationsGlossariesCreateRequest object.

  Fields:
    glossary: A Glossary resource to be passed as the request body.
    parent: Required. The project name.
  """

  glossary = _messages.MessageField('Glossary', 1)
  parent = _messages.StringField(2, required=True)


class TranslateProjectsLocationsGlossariesDeleteRequest(_messages.Message):
  r"""A TranslateProjectsLocationsGlossariesDeleteRequest object.

  Fields:
    name: Required. The name of the glossary to delete.
  """

  name = _messages.StringField(1, required=True)


class TranslateProjectsLocationsGlossariesGetRequest(_messages.Message):
  r"""A TranslateProjectsLocationsGlossariesGetRequest object.

  Fields:
    name: Required. The name of the glossary to retrieve.
  """

  name = _messages.StringField(1, required=True)


class TranslateProjectsLocationsGlossariesListRequest(_messages.Message):
  r"""A TranslateProjectsLocationsGlossariesListRequest object.

  Fields:
    filter: Optional. Filter specifying constraints of a list operation.
      Specify the constraint by the format of "key=value", where key must be
      "src" or "tgt", and the value must be a valid language code. For
      multiple restrictions, concatenate them by "AND" (uppercase only), such
      as: "src=en-US AND tgt=zh-CN". Notice that the exact match is used here,
      which means using 'en-US' and 'en' can lead to different results, which
      depends on the language code you used when you create the glossary. For
      the unidirectional glossaries, the "src" and "tgt" add restrictions on
      the source and target language code separately. For the equivalent term
      set glossaries, the "src" and/or "tgt" add restrictions on the term set.
      For example: "src=en-US AND tgt=zh-CN" will only pick the unidirectional
      glossaries which exactly match the source language code as "en-US" and
      the target language code "zh-CN", but all equivalent term set glossaries
      which contain "en-US" and "zh-CN" in their language set will be picked.
      If missing, no filtering is performed.
    pageSize: Optional. Requested page size. The server may return fewer
      glossaries than requested. If unspecified, the server picks an
      appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return. Typically, this is the value of
      [ListGlossariesResponse.next_page_token] returned from the previous call
      to `ListGlossaries` method. The first page is returned if `page_token`is
      empty or missing.
    parent: Required. The name of the project from which to list all of the
      glossaries.
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class TranslateProjectsLocationsListRequest(_messages.Message):
  r"""A TranslateProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class TranslateProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A TranslateProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class TranslateProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A TranslateProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class TranslateProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A TranslateProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class TranslateProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A TranslateProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class TranslateProjectsLocationsOperationsWaitRequest(_messages.Message):
  r"""A TranslateProjectsLocationsOperationsWaitRequest object.

  Fields:
    name: The name of the operation resource to wait on.
    waitOperationRequest: A WaitOperationRequest resource to be passed as the
      request body.
  """

  name = _messages.StringField(1, required=True)
  waitOperationRequest = _messages.MessageField('WaitOperationRequest', 2)


class TranslateProjectsLocationsRefineTextRequest(_messages.Message):
  r"""A TranslateProjectsLocationsRefineTextRequest object.

  Fields:
    parent: Required. Project or location to make a call. Must refer to a
      caller's project. Format: `projects/{project-number-or-
      id}/locations/{location-id}`. For global calls, use `projects/{project-
      number-or-id}/locations/global` or `projects/{project-number-or-id}`.
    refineTextRequest: A RefineTextRequest resource to be passed as the
      request body.
  """

  parent = _messages.StringField(1, required=True)
  refineTextRequest = _messages.MessageField('RefineTextRequest', 2)


class TranslateProjectsLocationsTranslateDocumentRequest(_messages.Message):
  r"""A TranslateProjectsLocationsTranslateDocumentRequest object.

  Fields:
    parent: Required. Location to make a regional call. Format:
      `projects/{project-number-or-id}/locations/{location-id}`. For global
      calls, use `projects/{project-number-or-id}/locations/global`. Non-
      global location is required for requests using AutoML models or custom
      glossaries. Models and glossaries must be within the same region (have
      the same location-id), otherwise an INVALID_ARGUMENT (400) error is
      returned.
    translateDocumentRequest: A TranslateDocumentRequest resource to be passed
      as the request body.
  """

  parent = _messages.StringField(1, required=True)
  translateDocumentRequest = _messages.MessageField('TranslateDocumentRequest', 2)


class TranslateProjectsLocationsTranslateTextRequest(_messages.Message):
  r"""A TranslateProjectsLocationsTranslateTextRequest object.

  Fields:
    parent: Required. Project or location to make a call. Must refer to a
      caller's project. Format: `projects/{project-number-or-id}` or
      `projects/{project-number-or-id}/locations/{location-id}`. For global
      calls, use `projects/{project-number-or-id}/locations/global` or
      `projects/{project-number-or-id}`. Non-global location is required for
      requests using AutoML models or custom glossaries. Models and glossaries
      must be within the same region (have same location-id), otherwise an
      INVALID_ARGUMENT (400) error is returned.
    translateTextRequest: A TranslateTextRequest resource to be passed as the
      request body.
  """

  parent = _messages.StringField(1, required=True)
  translateTextRequest = _messages.MessageField('TranslateTextRequest', 2)


class TranslateProjectsTranslateTextRequest(_messages.Message):
  r"""A TranslateProjectsTranslateTextRequest object.

  Fields:
    parent: Required. Project or location to make a call. Must refer to a
      caller's project. Format: `projects/{project-number-or-id}` or
      `projects/{project-number-or-id}/locations/{location-id}`. For global
      calls, use `projects/{project-number-or-id}/locations/global` or
      `projects/{project-number-or-id}`. Non-global location is required for
      requests using AutoML models or custom glossaries. Models and glossaries
      must be within the same region (have same location-id), otherwise an
      INVALID_ARGUMENT (400) error is returned.
    translateTextRequest: A TranslateTextRequest resource to be passed as the
      request body.
  """

  parent = _messages.StringField(1, required=True)
  translateTextRequest = _messages.MessageField('TranslateTextRequest', 2)


class TranslateTextGlossaryConfig(_messages.Message):
  r"""Configures which glossary should be used for a specific target language,
  and defines options for applying that glossary.

  Fields:
    contextualTranslationEnabled: Optional. If set to true, the glossary will
      be used for contextual translation.
    glossary: Required. Specifies the glossary used for this translation. Use
      this format: projects/*/locations/*/glossaries/*
    ignoreCase: Optional. Indicates match is case-insensitive. Default value
      is false if missing.
  """

  contextualTranslationEnabled = _messages.BooleanField(1)
  glossary = _messages.StringField(2)
  ignoreCase = _messages.BooleanField(3)


class TranslateTextRequest(_messages.Message):
  r"""The request message for synchronous translation.

  Messages:
    LabelsValue: Optional. The labels with user-defined metadata for the
      request. Label keys and values can be no longer than 63 characters
      (Unicode codepoints), can only contain lowercase letters, numeric
      characters, underscores and dashes. International characters are
      allowed. Label values are optional. Label keys must start with a letter.
      See https://cloud.google.com/translate/docs/labels for more information.

  Fields:
    contents: Required. The content of the input in string format. We
      recommend the total content be less than 30k codepoints. The max length
      of this field is 1024. Use BatchTranslateText for larger text.
    glossaryConfig: Optional. Glossary to be applied. The glossary must be
      within the same region (have the same location-id) as the model,
      otherwise an INVALID_ARGUMENT (400) error is returned.
    labels: Optional. The labels with user-defined metadata for the request.
      Label keys and values can be no longer than 63 characters (Unicode
      codepoints), can only contain lowercase letters, numeric characters,
      underscores and dashes. International characters are allowed. Label
      values are optional. Label keys must start with a letter. See
      https://cloud.google.com/translate/docs/labels for more information.
    mimeType: Optional. The format of the source text, for example,
      "text/html", "text/plain". If left blank, the MIME type defaults to
      "text/html".
    model: Optional. The `model` type requested for this translation. The
      format depends on model type: - AutoML Translation models:
      `projects/{project-number-or-id}/locations/{location-id}/models/{model-
      id}` - General (built-in) models: `projects/{project-number-or-
      id}/locations/{location-id}/models/general/nmt`, For global (non-
      regionalized) requests, use `location-id` `global`. For example,
      `projects/{project-number-or-id}/locations/global/models/general/nmt`.
      If not provided, the default Google model (NMT) will be used
    sourceLanguageCode: Optional. The BCP-47 language code of the input text
      if known, for example, "en-US" or "sr-Latn". Supported language codes
      are listed in [Language
      Support](https://cloud.google.com/translate/docs/languages). If the
      source language isn't specified, the API attempts to identify the source
      language automatically and returns the source language within the
      response.
    targetLanguageCode: Required. The BCP-47 language code to use for
      translation of the input text, set to one of the language codes listed
      in [Language
      Support](https://cloud.google.com/translate/docs/languages).
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. The labels with user-defined metadata for the request. Label
    keys and values can be no longer than 63 characters (Unicode codepoints),
    can only contain lowercase letters, numeric characters, underscores and
    dashes. International characters are allowed. Label values are optional.
    Label keys must start with a letter. See
    https://cloud.google.com/translate/docs/labels for more information.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  contents = _messages.StringField(1, repeated=True)
  glossaryConfig = _messages.MessageField('TranslateTextGlossaryConfig', 2)
  labels = _messages.MessageField('LabelsValue', 3)
  mimeType = _messages.StringField(4)
  model = _messages.StringField(5)
  sourceLanguageCode = _messages.StringField(6)
  targetLanguageCode = _messages.StringField(7)


class TranslateTextResponse(_messages.Message):
  r"""A TranslateTextResponse object.

  Fields:
    glossaryTranslations: Text translation responses if a glossary is provided
      in the request. This can be the same as `translations` if no terms
      apply. This field has the same length as `contents`.
    translations: Text translation responses with no glossary applied. This
      field has the same length as `contents`.
  """

  glossaryTranslations = _messages.MessageField('Translation', 1, repeated=True)
  translations = _messages.MessageField('Translation', 2, repeated=True)


class Translation(_messages.Message):
  r"""A single translation response.

  Fields:
    detectedLanguageCode: The BCP-47 language code of source text in the
      initial request, detected automatically, if no source language was
      passed within the initial request. If the source language was passed,
      auto-detection of the language does not occur and this field is empty.
    glossaryConfig: The `glossary_config` used for this translation.
    model: Only present when `model` is present in the request. `model` here
      is normalized to have project number. For example: If the `model`
      requested in TranslationTextRequest is `projects/{project-
      id}/locations/{location-id}/models/general/nmt` then `model` here would
      be normalized to `projects/{project-number}/locations/{location-
      id}/models/general/nmt`.
    translatedText: Text translated into the target language. If an error
      occurs during translation, this field might be excluded from the
      response.
  """

  detectedLanguageCode = _messages.StringField(1)
  glossaryConfig = _messages.MessageField('TranslateTextGlossaryConfig', 2)
  model = _messages.StringField(3)
  translatedText = _messages.StringField(4)


class WaitOperationRequest(_messages.Message):
  r"""The request message for Operations.WaitOperation.

  Fields:
    timeout: The maximum duration to wait before timing out. If left blank,
      the wait will be at most the time permitted by the underlying HTTP/RPC
      protocol. If RPC context deadline is also specified, the shorter one
      will be used.
  """

  timeout = _messages.StringField(1)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Create workflow template command."""


from googlecloudsdk.api_lib.dataproc import dataproc as dp
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataproc import flags
from googlecloudsdk.command_lib.dataproc import workflow_templates
from googlecloudsdk.command_lib.util.args import labels_util

DETAILED_HELP = {
    'EXAMPLES':
        """\
      To create a workflow template named ``my-workflow-template'' in region
      ``us-central1'' with label params 'key1'='value1' and 'key2'='value2', run:

        $ {command} my-workflow-template --region=us-central1 --labels="key1=value1,key2=value2"
      """,
}


class Create(base.CreateCommand):
  """Create a workflow template."""

  detailed_help = DETAILED_HELP

  @classmethod
  def Args(cls, parser):
    dataproc = dp.Dataproc(cls.ReleaseTrack())
    labels_util.AddCreateLabelsFlags(parser)
    workflow_templates.AddDagTimeoutFlag(parser, False)
    workflow_templates.AddKmsKeyFlag(parser, False)
    flags.AddTemplateResourceArg(parser, 'create', dataproc.api_version)

  def Run(self, args):
    dataproc = dp.Dataproc(self.ReleaseTrack())
    messages = dataproc.messages

    template_ref = args.CONCEPTS.template.Parse()
    # TODO(b/109837200) make the dataproc discovery doc parameters consistent
    # Parent() fails for the collection because of projectId/projectsId and
    # regionId/regionsId inconsistencies.
    # parent = template_ref.Parent().RelativePath()
    parent = '/'.join(template_ref.RelativeName().split('/')[0:4])

    workflow_template = messages.WorkflowTemplate(
        id=template_ref.Name(),
        name=template_ref.RelativeName(),
        labels=labels_util.ParseCreateArgs(
            args, messages.WorkflowTemplate.LabelsValue))

    if args.dag_timeout:
      workflow_template.dagTimeout = str(args.dag_timeout) + 's'

    if args.kms_key:
      workflow_template.encryptionConfig = (
          workflow_templates.GenerateEncryptionConfig(args.kms_key, dataproc)
      )

    request = messages.DataprocProjectsRegionsWorkflowTemplatesCreateRequest(
        parent=parent, workflowTemplate=workflow_template)

    template = dataproc.client.projects_regions_workflowTemplates.Create(
        request)
    return template



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The Start command."""


from googlecloudsdk.api_lib.app import appengine_api_client
from googlecloudsdk.api_lib.app import operations_util
from googlecloudsdk.api_lib.app import version_util
from googlecloudsdk.calliope import base
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.console import progress_tracker
from googlecloudsdk.core.resource import resource_printer


class VersionsStartError(exceptions.Error):
  """Errors occurring when starting versions."""
  pass


class Start(base.Command):
  """Start serving specified versions.

  This command starts serving the specified versions. It may only be used if the
  scaling module for your service has been set to manual.
  """

  detailed_help = {
      'EXAMPLES': """\
          To start a specific version across all services, run:

            $ {command} v1

          To start multiple named versions across all services, run:

            $ {command} v1 v2

          To start a single version on a single service, run:

            $ {command} --service=servicename v1

          To start multiple versions in a single service, run:

            $ {command} --service=servicename v1 v2
          """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument('versions', nargs='+', help=(
        'The versions to start. (optionally filtered by the --service flag).'))
    parser.add_argument('--service', '-s',
                        help=('If specified, only start versions from the '
                              'given service.'))

  def Run(self, args):
    # TODO(b/36052475): This fails with "module/version does not exist" even
    # when it exists if the scaling mode is set to auto.  It would be good
    # to improve that error message.
    api_client = appengine_api_client.GetApiClientForTrack(self.ReleaseTrack())
    services = api_client.ListServices()
    versions = version_util.GetMatchingVersions(
        api_client.ListVersions(services),
        args.versions, args.service)

    if not versions:
      log.warning('No matching versions found.')
      return

    fmt = 'list[title="Starting the following versions:"]'
    resource_printer.Print(versions, fmt, out=log.status)
    console_io.PromptContinue(cancel_on_no=True)

    errors = {}
    # Sort versions to make behavior deterministic enough for unit testing.
    for version in sorted(versions, key=str):
      try:
        with progress_tracker.ProgressTracker('Starting [{0}]'.format(version)):
          operations_util.CallAndCollectOpErrors(
              api_client.StartVersion, version.service, version.id)
      except operations_util.MiscOperationError as err:
        errors[version] = str(err)
    if errors:
      printable_errors = {}
      for version, error_msg in errors.items():
        short_name = '[{0}/{1}]'.format(version.service, version.id)
        printable_errors[short_name] = '{0}: {1}'.format(short_name, error_msg)
      raise VersionsStartError(
          'Issues starting version(s): {0}\n\n'.format(
              ', '.join(list(printable_errors.keys()))) +
          '\n\n'.join(list(printable_errors.values())))



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Datastream private connections API."""


from googlecloudsdk.api_lib.datastream import util
from googlecloudsdk.calliope import base
from googlecloudsdk.core import resources


def GetPrivateConnectionURI(resource):
  private_connection = resources.REGISTRY.ParseRelativeName(
      resource.name,
      collection='datastream.projects.locations.privateConnections')
  return private_connection.SelfLink()


class PrivateConnectionsClient:
  """Client for private connections service in the API."""

  def __init__(self, client=None, messages=None, location=None):
    self._client = client or util.GetClientInstance(location=location)
    self._messages = messages or util.GetMessagesModule()
    self._service = self._client.projects_locations_privateConnections
    self._resource_parser = util.GetResourceParser()

  def _GetPrivateConnection(self, private_connection_id, release_track, args):
    """Returns a private connection object."""
    private_connection_obj = self._messages.PrivateConnection(
        name=private_connection_id, labels={}, displayName=args.display_name)

    if hasattr(args, 'network_attachment') and args.network_attachment:
      private_connection_obj.pscInterfaceConfig = (
          self._messages.PscInterfaceConfig(
              networkAttachment=args.network_attachment
          )
      )
    else:
      # TODO(b/207467120): use only vpc flag.
      if release_track == base.ReleaseTrack.BETA:
        vpc_peering_ref = args.CONCEPTS.vpc_name.Parse()
      else:
        vpc_peering_ref = args.CONCEPTS.vpc.Parse()

      private_connection_obj.vpcPeeringConfig = self._messages.VpcPeeringConfig(
          vpc=vpc_peering_ref.RelativeName(), subnet=args.subnet
      )

    return private_connection_obj

  def Create(self, parent_ref, private_connection_id, release_track, args=None):
    """Creates a private connection.

    Args:
      parent_ref: a Resource reference to a parent datastream.projects.locations
        resource for this private connection.
      private_connection_id: str, the name of the resource to create.
      release_track: Some arguments are added based on the command release
        track.
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      Operation: the operation for creating the private connection.
    """

    private_connection = self._GetPrivateConnection(private_connection_id,
                                                    release_track, args)

    request_id = util.GenerateRequestId()
    create_req_type = (
        self._messages.DatastreamProjectsLocationsPrivateConnectionsCreateRequest
    )
    create_req = create_req_type(
        privateConnection=private_connection,
        privateConnectionId=private_connection.name,
        parent=parent_ref,
        requestId=request_id,
        validateOnly=args.validate_only,
    )

    return self._service.Create(create_req)



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Design Center Command Lib added Flags."""


import argparse
import textwrap

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import yaml


def GetProjectResourceSpec():
  return concepts.ResourceSpec(
      'designcenter.projects',
      resource_name='project',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )


def GetProjectResourceArg(
    arg_name='project',
    help_text=None,
    positional=False,
    required=True,
):
  """Constructs and returns the Project Resource Argument."""
  help_text = help_text or 'Project ID.'
  return concept_parsers.ConceptParser.ForResource(
      '{}{}'.format('' if positional else '--', arg_name),
      GetProjectResourceSpec(),
      help_text,
      required=required,
  )


def LocationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='location',
      help_text='The Cloud location for the {resource}.',
  )


def GetLocationResourceSpec():
  return concepts.ResourceSpec(
      'designcenter.projects.locations',
      resource_name='location',
      locationsId=LocationAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )


def GetLocationResourceArg(
    arg_name='location',
    help_text=None,
    positional=False,
    required=True,
):
  """Constructs and returns the Location Resource Argument."""
  help_text = help_text or 'Location.'

  return concept_parsers.ConceptParser.ForResource(
      '{}{}'.format('' if positional else '--', arg_name),
      GetLocationResourceSpec(),
      help_text,
      required=required,
  )


def SpaceResourceAttributeConfig(arg_name, help_text):
  """Helper function for constructing ResourceAttributeConfig."""
  return concepts.ResourceParameterAttributeConfig(
      name=arg_name,
      help_text=help_text,
  )


def GetSpaceResourceSpec(arg_name='space', help_text=None):
  """Constructs and returns the Resource specification for Space."""
  return concepts.ResourceSpec(
      'designcenter.projects.locations.spaces',
      resource_name='space',
      spacesId=SpaceResourceAttributeConfig(arg_name, help_text),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
  )


def GetSpaceResourceArg(
    arg_name='space', help_text=None, positional=True, required=True
):
  """Constructs and returns the Space ID Resource Argument."""
  help_text = help_text or 'The Space ID.'

  return concept_parsers.ConceptParser.ForResource(
      '{}{}'.format('' if positional else '--', arg_name),
      GetSpaceResourceSpec(arg_name, help_text),
      help_text,
      required=required,
  )


# -- Revision Resource and Flags --


def CatalogAttributeConfig():
  """Creates an attribute config for the catalog."""
  return concepts.ResourceParameterAttributeConfig(
      name='catalog',
      help_text='The ID of the catalog.'
  )


def CatalogTemplateAttributeConfig():
  """Creates an attribute config for the template."""
  return concepts.ResourceParameterAttributeConfig(
      name='template',
      help_text='The ID of the template.'
  )


def GetCatalogTemplateRevisionResourceSpec():
  """Creates the resource spec for a revision."""
  return concepts.ResourceSpec(
      'designcenter.projects.locations.spaces.catalogs.templates.revisions',
      resource_name='revision',
      revisionsId=concepts.ResourceParameterAttributeConfig(
          name='revision', help_text='The ID of the revision to create.'
      ),
      templatesId=CatalogTemplateAttributeConfig(),
      catalogsId=CatalogAttributeConfig(),
      spacesId=SpaceResourceAttributeConfig('space', 'The ID of the space.'),
      locationsId=LocationAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )


def AddDescribeLocationFlags(parser):
  GetLocationResourceArg(positional=True).AddToParser(parser)


def AddGetIamPolicyFlags(parser):
  GetSpaceResourceArg().AddToParser(parser)


def AddSetIamPolicyFlags(parser):
  GetSpaceResourceArg().AddToParser(parser)


def AddTestIamPermissionsFlags(parser):
  GetSpaceResourceArg().AddToParser(parser)


def AddCreateCatalogTemplateRevisionFlags(parser):
  """Adds all flags for the create revision command.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in tests.
  """
  # This defines the resource argument for the revision.
  # By using the full resource spec and a positional name ('revision'), the
  # SDK automatically creates a positional argument for the revision ID and
  # flags for all its parents (--template, --catalog, --space, etc.).
  concept_parsers.ConceptParser.ForResource(
      'revision',
      GetCatalogTemplateRevisionResourceSpec(),
      'The revision to create.',
      required=True,
  ).AddToParser(parser)
  parser.add_argument('--description', help='A description for the revision.')

  source_group = parser.add_group(mutex=True, required=True)

  dev_connect_group = source_group.add_group(
      help='Flags for Developer Connect source.'
  )
  dev_connect_group.add_argument(
      '--developer-connect-repo',
      help=(
          'The Developer Connect repository to use as a source. Example: '
          '`projects/my-project/locations/us-central1/connections/my-connection/'
          'gitRepositoryLinks/my-repo`'
      ),
      required=True,
  )
  dev_connect_group.add_argument(
      '--developer-connect-repo-ref',
      help=(
          'The Git ref (branch or tag) within the repository to use. Example:'
          ' `"refs/tags/v1.0.0"` or `"refs/heads/main"` or '
          '`"refs/commits/269b518b99d06b31ff938a2d182e75f5e41941c7"`.'
      ),
      required=True,
  )
  dev_connect_group.add_argument(
      '--developer-connect-repo-dir',
      help=(
          'The directory within the repository to use. Example:'
          ' `"modules/my-product"`'
      ),
      required=True,
  )

  git_source_group = source_group.add_group(
      help='Flags for Git source.'
  )
  git_source_group.add_argument(
      '--git-source-repo',
      help='Git repository for Git source. Example:'
      ' `GoogleCloudPlatform/terraform-google-cloud-run`',
      required=True,
  )
  git_source_group.add_argument(
      '--git-source-ref-tag',
      help='Git reference tag for Git source. Example: `"v1.0.0"`',
      required=True,
  )
  git_source_group.add_argument(
      '--git-source-dir',
      help=(
          'Git directory for Git source. Example: `"modules/my-product"`.'
          ' This field is optional.'
      ),
      required=False,
  )

  source_group.add_argument(
      '--application-template-revision-source',
      help=(
          'Application template revision to use as source. Example:'
          ' `projects/my-project/locations/us-central1/spaces/my-space/catalogs/my-catalog/templates/my-template/revisions/r1`'
      ),
      required=False,
  )

  source_group.add_argument(
      '--gcs-source-uri',
      help=(
          'Google Cloud Storage URI for source. Example:'
          ' `gs://my-bucket/my-template`.'
      ),
      required=False,
  )

  oci_repo_group = source_group.add_group(help='Flags for OCI Repo source.')
  oci_repo_group.add_argument(
      '--oci-repo-uri',
      help=(
          'OCI Repo URI for OCI Repo source. Example:'
          ' `oci://us-west1-docker.pkg.dev/my-project/my-repo/my-chart`'
      ),
      required=True,
  )
  oci_repo_group.add_argument(
      '--oci-repo-version',
      help=(
          'OCI Repo version for OCI Repo source. Example: `"1.0.0"`. This field'
          ' is optional.'
      ),
      required=False,
  )

  parser.add_argument(
      '--metadata',
      type=arg_parsers.YAMLFileContents(),
      help=(
          'Path to a local YAML file containing the template metadata. Example:'
          ' `"path/to/metadata.yaml"`.'
      ),
  )


def AddRegisterWithApphubFlags(parser):
  """Adds all flags for the register with apphub command.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in tests.
  """
  GetSpaceResourceArg(
      arg_name='space',
      help_text='The ID of the space.',
  ).AddToParser(parser)
  source_group = parser.add_group(mutex=True, required=True)
  source_group.add_argument(
      '--apphub-application-uri',
      help=(
          'AppHub application URI to register the deployed resource using'
          ' application template as source in the space.'
          ' Format: `projects/{projectId}/locations/{locationId}/applications/'
          '{applicationId}'
      ),
  )
  source_group.add_argument(
      '--adc-application-uri',
      help=(
          'The Application Design Center application URI to register the'
          ' deployed resources with the AppHub application using Application'
          ' Design Center application as source in the space. Format:'
          ' `projects/{projectId}/locations/{locationId}/spaces/{spaceId}/applications/'
          '{applicationId}`'
      ),
  )
  parser.add_argument(
      '--tfstate-location',
      help='Path to the Terraform state file (e.g., `terraform.tfstate`).',
  )
  base.ASYNC_FLAG.AddToParser(parser)


def AddRegisterAppHubApplicationResourcesFlags(parser: argparse.ArgumentParser) -> None:
  """Adds all flags for the register app hub application resources command.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in tests.
  """
  concept_parsers.ConceptParser.ForResource(
      '--location',
      GetLocationResourceSpec(),
      'The location to register resources in.',
      required=True,
  ).AddToParser(parser)
  tfstate_group = parser.add_group(mutex=True)
  tfstate_group.add_argument(
      '--tfstate-content',
      required=False,
      help=textwrap.dedent("""\
          The Terraform state (tfstate) content as a raw string.
          For large state files exceeding 10MB, use the `tfstate-signed-gcs-uri`
          field instead."""),
  )
  tfstate_group.add_argument(
      '--tfstate-signed-gcs-uri',
      required=False,
      help=textwrap.dedent("""\
          A securely signed Cloud Storage URI pointing to the tfstate file.
          The URI must be signed to grant the service temporary read access to the
          state file. ADC imposes a limit on the maximum size of the state file
          accessed via this URI."""),
  )
  parser.add_argument(
      '--apphub-application',
      required=True,
      help="""The name of the AppHub Application.
      Format:
      `projects/{project}/locations/{location}/applications/{application}`""",
  )
  base.ASYNC_FLAG.AddToParser(parser)


def AddTfStateSourceFlags(parser):
  """Adds flags for specifying the Terraform state source.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in tests.
  """
  tfstate_source_group = parser.add_group(mutex=True, required=True)
  tfstate_source_group.add_argument(
      '--terraform-state',
      help=(
          'The Terraform state (tfstate) content as a raw JSON string.'
          ' Example: \'{"version":4, "resources": [...]}\''
      ),
  )
  tfstate_source_group.add_argument(
      '--tfstate-signed-gcs-uri',
      help=(
          'A securely signed Cloud Storage URI pointing to the tfstate file.'
          ' Example: `https://storage.googleapis.com/my-bucket/tfstate.json'
          '?x-goog-signature=...`'
      ),
  )


def AddServiceAccountFlag(parser):
  """Adds the service account flag.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in tests.
  """
  parser.add_argument(
      '--service-account',
      help=(
          'The email address of the service account to use for this operation.'
          ' Format: `projects/{PROJECT}/serviceAccounts/{EMAIL_ADDRESS}`'
      ),
      required=False,
  )


def AddRegisterDeployedApplicationFlags(parser):
  """Adds flags for the register deployed application command.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in tests.
  """
  concept_parsers.ConceptParser.ForResource(
      'APPLICATION',
      GetApplicationResourceSpec(),
      'The application with which the resources will be registered.',
      required=True).AddToParser(parser)
  AddTfStateSourceFlags(parser)
  AddServiceAccountFlag(parser)
  base.ASYNC_FLAG.AddToParser(parser)


def AddRegisterDeployedResourcesFlags(parser):
  """Adds flags for the register deployed resources command.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in tests.
  """
  GetSpaceResourceArg(
      arg_name='space',
      help_text='The parent space.',
  ).AddToParser(parser)
  parser.add_argument(
      '--apphub-application',
      help=(
          'The name of the AppHub Application. Format:'
          ' `projects/{project}/locations/{location}/applications/{application}`'
      ),
      required=True,
  )
  AddTfStateSourceFlags(parser)
  AddServiceAccountFlag(parser)
  base.ASYNC_FLAG.AddToParser(parser)


def AddInferConnectionsFlags(parser):
  """Adds all flags for the infer connections command.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in tests.
  """
  base.ASYNC_FLAG.AddToParser(parser)
  GetSpaceResourceArg('space', 'The space ID.').AddToParser(parser)
  parser.add_argument(
      '--catalog-template-revision-uris',
      type=arg_parsers.ArgList(),
      metavar='CATALOG_TEMPLATE_REVISION_URI',
      help=(
          'A comma-separated list of catalog template revision URIs to infer'
          ' connections for. If not provided, the system infers connections'
          ' for the latest revisions of all catalog templates in all the'
          ' catalogs present in the space.'
          ' Format:'
          ' `projects/{projectId}/locations/{locationId}/spaces/{spaceId}/'
          'catalogs/{catalogId}/templates/{templateId}/revisions/{revisionId}`'
      ),
  )
  parser.add_argument(
      '--use-gemini',
      action='store_true',
      default=False,
      help='Use Gemini to infer connections.',
  )


def GetApplicationTemplateResourceSpec(arg_name='application_template_id'):
  """Constructs and returns the Resource specification for Application Template."""
  return concepts.ResourceSpec(
      'designcenter.projects.locations.spaces.applicationTemplates',
      resource_name='application_template',
      applicationTemplatesId=concepts.ResourceParameterAttributeConfig(
          name=arg_name, help_text='The ID of the application template.'
      ),
      spacesId=SpaceResourceAttributeConfig('space', 'The ID of the space.'),
      locationsId=LocationAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )


def AddApplicationTemplateResourceArg(parser, verb):
  """Adds the Application Template resource argument to the given parser."""
  concept_parsers.ConceptParser.ForResource(
      'APPLICATION_TEMPLATE',
      GetApplicationTemplateResourceSpec(),
      'The application template {} IaC.'.format(verb),
      required=True).AddToParser(parser)


def GetApplicationResourceSpec(arg_name='application'):
  """Constructs and returns the Resource specification for Application."""
  return concepts.ResourceSpec(
      'designcenter.projects.locations.spaces.applications',
      resource_name='application',
      applicationsId=concepts.ResourceParameterAttributeConfig(
          name=arg_name, help_text='The ID of the application.'
      ),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      spacesId=SpaceResourceAttributeConfig('space', 'The ID of the space.'),
  )


def AddApplicationResourceArg(parser, verb):
  """Adds the Application resource argument to the given parser."""
  concept_parsers.ConceptParser.ForResource(
      'APPLICATION',
      GetApplicationResourceSpec(),
      'The application {} IaC.'.format(verb),
      required=True).AddToParser(parser)


def AddImportIacFlags(parser):
  """Adds flags for import-iac commands to the given parser."""
  source_group = parser.add_mutually_exclusive_group(required=True)
  source_group.add_argument(
      '--gcs-uri',
      help=('The Cloud Storage URI of the Terraform code '
            '(e.g., gs://my-bucket/iac).'),
  )
  source_group.add_argument(
      '--iac-module-from-file',
      type=arg_parsers.FileContents(),
      help=('Path to a local YAML or JSON file containing the IaC module '
            'definition.'),
  )
  parser.add_argument(
      '--allow-partial-import',
      action='store_true',
      default=False,
      help=('If set, partially import valid IaC changes and ignore invalid '
            'ones.'),
  )
  parser.add_argument(
      '--validate-iac',
      action='store_true',
      default=False,
      help='Validate the IaC without performing the import.',
  )


def AddDeployApplicationFlags(parser):
  """Adds all flags for the deploy application command.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in tests.
  """
  parser.add_argument(
      '--replace',
      action='store_true',
      default=False,
      help=(
          'Flag to update the existing deployment. If not set or false, deploy'
          ' will fail if application state is in the DEPLOYED state.'
      ),
  )
  parser.add_argument(
      '--worker-pool',
      type=arg_parsers.RegexpValidator(
          r'projects/[^/]+/locations/[^/]+/workerPools/[^/]+',
          'The worker pool resource name must be in the format: '
          '`projects/{project}/locations/{location}/workerPools/{workerPoolId}`',
      ),
      help=textwrap.dedent("""\
          The user-specified Worker Pool resource in which the Cloud Build job
          will execute. Format:
          projects/{project}/locations/{location}/workerPools/{workerPoolId}
          If this flag is omitted, the worker pool already defined on the
          application will be used. If no worker pool is defined on the
          application, the default Cloud Build worker pool is used. The
          worker pool must exist in the same region as the application.\
      """),
  )

  parser.add_argument(
      '--service-account',
      help=textwrap.dedent('''\
          The service account to use for this deployment.

          * If provided, this service account will be used to execute the
            deployment process, taking precedence over any service_account
            specified on the Application resource.
          * The caller must have the "iam.serviceAccounts.actAs" permission on
            this service account.
          * If this field is omitted, the system will use the "service_account"
            defined within the Application resource.
          * If this field is omitted with --create-sa flag, the system will create a new and unique service_account
            and use it for the deployment.
          * We recommend that you provide a service account here or on the
            Application resource. If you don't provide a service account, the
            deployment will fail.
          * If the `--create-sa` flag is also provided, this value is the ID of
            a new service account to be created (e.g., `my-new-sa`).

          Format: `projects/{PROJECT}/serviceAccounts/{EMAIL_ADDRESS}` (when not
          using `--create-sa`)
          '''),
  )
  parser.add_argument(
      '--create-sa',
      action='store_true',
      help='Create a new service account for the deployment.',
  )


def ParseIacModuleFile(file_contents):
  """Parses YAML or JSON file contents into a Python dict."""
  try:
    return yaml.load(file_contents)
  except Exception as e:
    raise arg_parsers.ArgumentTypeError(
        'Failed to parse IaC module file: {}'.format(e))


def AddPreviewApplicationFlags(parser):
  """Adds all flags for the preview application command.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in tests.
  """
  parser.add_argument(
      '--worker-pool',
      type=arg_parsers.RegexpValidator(
          r'projects/[^/]+/locations/[^/]+/workerPools/[^/]+',
          'The worker pool resource name must be in the format: '
          '`projects/{project}/locations/{location}/workerPools/{workerPoolId}`',
      ),
      help=textwrap.dedent("""\
          The user-specified Worker Pool resource in which the Cloud Build job
          will execute. Format:
          projects/{project}/locations/{location}/workerPools/{workerPoolId}
          If this flag is omitted, the worker pool already defined on the
          application will be used. If no worker pool is defined on the
          application, the default Cloud Build worker pool is used. The
          worker pool must exist in the same region as the application.\
      """),
  )

  parser.add_argument(
      '--service-account',
      help=textwrap.dedent('''\
          The service account to use for this preview.

          * If provided, this service account will be used to execute the
            preview process, taking precedence over any service_account
            specified on the Application resource.
          * The caller must have the "iam.serviceAccounts.actAs" permission on
            this service account.
          * If this field is omitted, the system will use the "service_account"
            defined within the Application resource.
          * If this field is omitted with --create-sa flag, the system will create a new and unique service_account
            and use it for the preview.
          * We recommend that you provide a service account here or on the
            Application resource. If you don't provide a service account, the
            preview will fail.
          * If the `--create-sa` flag is also provided, this value is the ID of
            a new service account to be created (e.g., `my-new-sa`).

          Format: `projects/{PROJECT}/serviceAccounts/{EMAIL_ADDRESS}` (when not
          using `--create-sa`)
          '''),
  )
  parser.add_argument(
      '--create-sa',
      action='store_true',
      help='Create a new service account for the preview.',
  )


def AddGenerateAssessmentReportFlags(parser):
  """Adds flags for GenerateAssessmentReport command."""
  parser.add_argument(
      '--service-account',
      help=textwrap.dedent('''\
          The full resource name of the service account to use for this
          assessment report generation operation.

          * This service account will be used to execute the assessment report
            generation process.
          * The caller must have the "iam.serviceAccounts.actAs" permission on
            this service account.
          Format: `projects/{PROJECT}/serviceAccounts/{EMAIL_ADDRESS}`
          '''),
      required=False,
  )


def AddGenerateTerraformAssessmentReportFlags(parser):
  """Adds flags for GenerateTerraformAssessmentReport command.

  Args:
    parser: An argparse.ArgumentParser-like object. It is mocked out in tests.
  """
  GetSpaceResourceArg(positional=True).AddToParser(parser)

  parser.add_argument(
      '--terraform-plan',
      type=arg_parsers.FileContents(binary=True),
      required=True,
      help='Path to the Terraform plan file (JSON format).',
  )

  parser.add_argument(
      '--additional-frameworks',
      type=arg_parsers.ArgList(),
      metavar='FRAMEWORK',
      help='Additional frameworks to run assessment against.',
  )



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of list command for batch job bucket operations."""

from googlecloudsdk.api_lib.storage import storage_batch_operations_api
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage.batch_operations.jobs import resource_args


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class List(base.ListCommand):
  """List bucket operations for a batch operation job."""

  hints = base.CommandHint(read_only=True)

  detailed_help = {
      'DESCRIPTION': """
      List bucket operations for a specific batch operation job.
      """,
      'EXAMPLES': """
      The following example command lists bucket operations for batch job `my-job`:

          $ {command} --job=my-job

      The following example command lists bucket operations for batch job `my-job`
      filtering by bucket name `my-bucket`:

          $ {command} --job=my-job --buckets=my-bucket
      """,
  }

  @staticmethod
  def Args(parser):
    resource_args.add_batch_job_flag_resource_arg(
        parser, 'for which to list bucket operations'
    )
    base.URI_FLAG.RemoveFromParser(parser)
    parser.add_argument(
        '--buckets',
        type=arg_parsers.ArgList(),
        metavar='BUCKETS',
        help='If provided, only show operations for buckets in the list.',
    )
    parser.display_info.AddFormat("""
      table(
        name.basename():label=BUCKET_OPERATION_ID,
        bucketName:label=BUCKET,
        counters:wrap=20:label=COUNTERS,
        errorSummaries:wrap=20:label=ERROR_SUMMARIES,
        state:wrap=20:label=STATE,
        createTime.date():label=CREATE_TIME,
        completeTime.date():label=COMPLETION_TIME
      )
    """)

  def Run(self, args):
    job_ref = args.CONCEPTS.job.Parse()
    return storage_batch_operations_api.StorageBatchOperationsApi().list_bucket_operations(
        batch_job_name=job_ref.RelativeName(),
        bucket_names=args.buckets,
        page_size=args.page_size,
    )
